"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const querystring = require("querystring");
const url = require("url");
const util_1 = require("util");
const default404Handler = {
    handler: async (req, res) => {
        res.statusCode = 404;
        res.end();
    },
    parseBody: false,
};
class SlimJim {
    constructor() {
        const lib = http;
        this.server = lib.createServer(async (req, res) => {
            const urlInfo = url.parse(req.url, true);
            const handler = this.pathMatch(req.method, urlInfo.pathname);
            const parsedObj = {
                body: null,
                headers: req.headers,
                normalizedHeaders: req.headers,
                pathParameters: {},
                queryString: urlInfo.query,
                rawBody: null,
                requestContext: {
                    httpMethod: req.method,
                    resourcePath: req.url,
                },
            };
            if (!handler.parseBody) {
                const resObj = await handler.handler(req, res, parsedObj);
                if (util_1.isObject(resObj)) {
                    res.setHeader("Content-Type", "application/json");
                    res.end(JSON.stringify(resObj));
                }
                else {
                    res.end(resObj);
                }
            }
            else {
                const inputBuffer = [];
                req.on("data", (chunk) => {
                    inputBuffer.push(chunk);
                });
                req.on("end", async () => {
                    const rawBody = (inputBuffer.length === 1) ? inputBuffer[0].toString("utf8") :
                        Buffer.concat(inputBuffer).toString("utf8");
                    const contentType = req.headers["content-type"];
                    const bodyContent = attemptParse(contentType, rawBody);
                    parsedObj.rawBody = rawBody;
                    parsedObj.body = bodyContent;
                    const resObj = await handler.handler(req, res, parsedObj);
                    if (util_1.isObject(resObj)) {
                        res.setHeader("Content-Type", "application/json");
                        res.end(JSON.stringify(resObj));
                    }
                    else {
                        res.end(resObj);
                    }
                });
            }
        });
        this.handlerMap = {
            GET: {},
            POST: {},
        };
    }
    registerEndpoint(method, route, handler, parseBody) {
        const handlerData = {
            handler,
            parseBody,
        };
        switch (method) {
            case "GET":
                this.handlerMap.GET[route] = handlerData;
                break;
            case "POST":
                this.handlerMap.POST[route] = handlerData;
                break;
            default:
                throw new Error("Unsupported method");
        }
    }
    get(route, handler) {
        return this.registerEndpoint("GET", route, handler, false);
    }
    post(route, handler, parseBody) {
        return this.registerEndpoint("POST", route, handler, parseBody);
    }
    listen(port, cb) {
        this.server.listen(port, cb);
    }
    pathMatch(method, route) {
        let routeMap = null;
        let handler = null;
        switch (method) {
            case "GET":
                routeMap = this.handlerMap.GET;
                break;
            case "POST":
                routeMap = this.handlerMap.POST;
                break;
            default:
                return default404Handler;
        }
        handler = routeMap[route];
        return (handler != null) ? handler : default404Handler;
    }
}
exports.SlimJim = SlimJim;
function attemptParse(contentType, obj) {
    if (contentType == null) {
        return obj;
    }
    try {
        switch (contentType) {
            case "application/json":
                return JSON.parse(obj);
            case "application/x-www-form-urlencoded":
                return querystring.parse(obj);
            default:
                return obj;
        }
    }
    catch (err) {
        return obj;
    }
}
//# sourceMappingURL=slimjim.js.map