"use strict";
import * as http from "http";
import * as querystring from "querystring";
import * as url from "url";
import {isObject} from "util";

// const uws = require("uws");

export type HttpMethod = "GET" | "POST";

export type HttpHandler = (request: http.IncomingMessage, response: http.ServerResponse, requestData: IRequestData)
	=> Promise<any>;

interface IHandlerData {
	handler: HttpHandler;
	parseBody: boolean;
}

export interface ILookupTable {
	[key: string]: string;
}

export interface IRequestData {
	requestContext: {
		resourcePath: string;
		httpMethod: HttpMethod;
	};
	queryString: ILookupTable;
	body?: any;
	rawBody?: any;
	headers: http.IncomingHttpHeaders;
	normalizedHeaders: http.IncomingHttpHeaders;
	pathParameters: ILookupTable;
}

interface IRouteMap {
	[route: string]: IHandlerData;
}

interface IHandlerMap {
	GET: IRouteMap;
	POST: IRouteMap;
}

const default404Handler: IHandlerData = {
	handler: async (req, res) => {
		res.statusCode = 404;
		res.end();
	},
	parseBody: false,
};

export class SlimJim {
	private server: http.Server;
	private handlerMap: IHandlerMap;

	constructor() {
		const lib = http;

		this.server = lib.createServer(async (req: http.IncomingMessage, res: http.ServerResponse) => {
			const urlInfo = url.parse(req.url, true);
			const handler = this.pathMatch(req.method, urlInfo.pathname);
			const parsedObj: IRequestData = {
				body: null,
				headers: req.headers,
				normalizedHeaders: req.headers,
				pathParameters: {},
				queryString: urlInfo.query,
				rawBody: null,
				requestContext: {
					httpMethod: req.method as HttpMethod,
					resourcePath: req.url,
				},
			};
			if (!handler.parseBody) {
				const resObj: any = await handler.handler(req, res, parsedObj);
				if (isObject(resObj)) {
					res.setHeader("Content-Type", "application/json");
					res.end(JSON.stringify(resObj));
				} else {
					res.end(resObj);
				}
			} else {
				const inputBuffer: Buffer[] = [];
				req.on("data", (chunk: Buffer) => {
					inputBuffer.push(chunk);
				});
				req.on("end", async () => {
					const rawBody = (inputBuffer.length === 1) ? inputBuffer[0].toString("utf8") :
						Buffer.concat(inputBuffer).toString("utf8");
					const contentType = req.headers["content-type"] as string;
					const bodyContent = attemptParse(contentType, rawBody);
					parsedObj.rawBody = rawBody;
					parsedObj.body = bodyContent;

					const resObj: any = await handler.handler(req, res, parsedObj);
					if (isObject(resObj)) {
						res.setHeader("Content-Type", "application/json");
						res.end(JSON.stringify(resObj));
					} else {
						res.end(resObj);
					}
				});
			}
		});
		this.handlerMap = {
			GET: {},
			POST: {},
		};
	}

	public registerEndpoint(method: HttpMethod, route: string, handler: HttpHandler, parseBody: boolean) {
		const handlerData: IHandlerData = {
			handler,
			parseBody,
		};

		switch (method) {
			case "GET":
				this.handlerMap.GET[route] = handlerData;
				break;
			case "POST":
				this.handlerMap.POST[route] = handlerData;
				break;
			default:
				throw new Error("Unsupported method");
		}
	}

	public get(route: string, handler: HttpHandler) {
		return this.registerEndpoint("GET", route, handler, false);
	}

	public post(route: string, handler: HttpHandler, parseBody: boolean) {
		return this.registerEndpoint("POST", route, handler, parseBody);
	}

	public listen(port: number|string, cb: () => void): void {
		this.server.listen(port, cb);
	}

	private pathMatch(method: string, route: string): IHandlerData {
		let routeMap: { [route: string]: IHandlerData } = null;
		let handler: IHandlerData = null;
		switch (method) {
			case "GET":
				routeMap = this.handlerMap.GET;
				break;
			case "POST":
				routeMap = this.handlerMap.POST;
				break;
			default:
				return default404Handler;
		}
		handler = routeMap[route];
		return (handler != null) ? handler : default404Handler;
	}
}

function attemptParse(contentType: string, obj: any): any {
	if (contentType == null) {
		return obj;
	}
	try {
		switch (contentType) {
			case "application/json":
				return JSON.parse(obj);
			case "application/x-www-form-urlencoded":
				return querystring.parse(obj);
			default:
				return obj;
		}
	} catch (err) {
		return obj;
	}
}
